/**
 * This class contains information about a client of a reward program.
 * <p>
 * The client has a card with their name and card number on it. The card holder
 * can accumulate and redeem reward points.
 * 
 * @author Alan Williams
 */

public class CardHolder
{
	/**
	 * Name of the card holder.
	 */
	private String name;

	/**
	 * Number on the card.
	 */
	private int cardNumber;

	/**
	 * Number of points accumulated.
	 */
	private int numberOfPoints;

	/**
	 * Constructs a new CardHolder with a specified name and card number.
	 * <p>
	 * The number of reward points is always set to zero.
	 * 
	 * @param clientName
	 *            The card holder's name
	 * @param theCardNumber
	 *            Number on the card
	 */
	public CardHolder( String clientName, int theCardNumber )
	{
		this.name = clientName;
		this.cardNumber = theCardNumber;
		this.numberOfPoints = 0;
	}

	/**
	 * Returns the name of the card holder.
	 * 
	 * @return A string with the card holder's name.
	 */
	public String getName( )
	{
		return this.name;
	}

	/**
	 * Returns the card number.
	 * 
	 * @return An <code>int</code> representing the card number.
	 */
	public int getCardNumber( )
	{
		return this.cardNumber;
	}

	/**
	 * Private accessor for the number of points.
	 * 
	 * @return An <code>int</code> containing the number of points
	 *         accumulated.
	 */
	private int getNumberOfPoints( )
	{
		return this.numberOfPoints;
	}

	/**
	 * Modifies the number of points stored on the card.
	 * 
	 * @param points
	 *            The new value of points.
	 */
	public void setNumberOfPoints( int points )
	{
		this.numberOfPoints = points;
	}

	/**
	 * Adds points to the total already on the card.
	 * <p>
	 * If a negative value is specified for the parameter, that number of points
	 * is deducted from the points accumulated.
	 * 
	 * @param points
	 *            Amount by which to adjust the points total (positive or
	 *            negative).
	 */
	public void adjustPoints( int points )
	{
		this.numberOfPoints = this.numberOfPoints + points;
	}

	/**
	 * Determines if a specified card holder has at least the specified number
	 * of points.
	 * 
	 * @param aClient
	 *            The card holder for which to verify the points.
	 * @param numPoints
	 *            The number of points to be confirmed.
	 * @return True if the card holder has at least as many points as specified,
	 *         and false otherwise.
	 */
	public static boolean enoughPoints( CardHolder aClient, int numPoints )
	{
		return ( aClient.getNumberOfPoints( ) >= numPoints );
	}
}
