import junit.framework.TestCase;
import junit.framework.Assert;

/**
 * 
 */

/**
 * @author Alan Williams
 */
public class TransactionTest extends TestCase
{

	/**
	 * Test method for 'Transaction.Transaction(CardHolder)'.
	 */
	public void testTransaction( )
	{
		CardHolder ch = new CardHolder( "Alan", 123456 );

		Transaction t = new Transaction( ch );

		String nameExpected = new String( "Alan" );
		String nameActual = t.getCardHolderName( );
		Assert.assertEquals( nameExpected, nameActual );

		double amountPaidExpected = 0.0;
		double amountPaidActual = t.getAmountPaid( );
		Assert.assertEquals( amountPaidExpected, amountPaidActual, 0.0001 );

		int pointsExpected = 0;
		int pointsActual = t.getPoints( );
		Assert.assertEquals( pointsExpected, pointsActual );

		int categoryExpected = Transaction.INCOMPLETE;
		int categoryActual = t.getCategory( );
		Assert.assertEquals( categoryExpected, categoryActual );

		int saveTransactionNumber = t.getTransactionNumber( );

		Transaction t2 = new Transaction( ch );
		int transNumberExpected = saveTransactionNumber + 1;
		int transNumberActual = t2.getTransactionNumber( );
		Assert.assertEquals( transNumberExpected, transNumberActual );

	}

	/**
	 * Test method for 'Transaction.getCategory()' with purchase transaction.
	 */
	public void testGetCategoryPurchase( )
	{
		CardHolder ch = new CardHolder( "Alan", 123456 );
		Transaction t = new Transaction( ch );

		int categoryExpected = Transaction.INCOMPLETE;
		int categoryActual = t.getCategory( );
		Assert.assertEquals( categoryExpected, categoryActual );

		Product p1 = new Product( "Product 1", 1.23, 50 );
		Product p2 = new Product( "Product 2", 4.56, 75 );
		Product[] pList = new Product[] { p1, p2 };

		t.purchase( pList );

		categoryExpected = Transaction.PURCHASE;
		categoryActual = t.getCategory( );
		Assert.assertEquals( categoryExpected, categoryActual );

	}

	/**
	 * Test method for 'Transaction.getCategory()' with redemption transaction.
	 */
	public void testGetCategoryRedemption( )
	{
		CardHolder ch = new CardHolder( "Alan", 123456 );
		Transaction t = new Transaction( ch );

		int categoryExpected = Transaction.INCOMPLETE;
		int categoryActual = t.getCategory( );
		Assert.assertEquals( categoryExpected, categoryActual );

		ch.setNumberOfPoints( 50000 );

		Product p1 = new Product( "Product 1", 1.23, 50 );
		Product p2 = new Product( "Product 2", 4.56, 75 );
		Product[] pList = new Product[] { p1, p2 };

		t.redeemPoints( pList );

		categoryExpected = Transaction.REDEMPTION;
		categoryActual = t.getCategory( );
		Assert.assertEquals( categoryExpected, categoryActual );

	}

	/**
	 * Test method for 'Transaction.getTransactionNumber()'.
	 */
	public void testGetTransactionNumber( )
	{
		CardHolder ch = new CardHolder( "Alan", 123456 );
		Transaction t1 = new Transaction( ch );
		Transaction t2 = new Transaction( ch );
		Transaction t3 = new Transaction( ch );

		int expectedNumber = t1.getTransactionNumber( ) + 1;
		int actualNumber = t2.getTransactionNumber( );
		Assert.assertEquals( expectedNumber, actualNumber );

		expectedNumber = t2.getTransactionNumber( ) + 1;
		actualNumber = t3.getTransactionNumber( );
		Assert.assertEquals( expectedNumber, actualNumber );
	}

	/**
	 * Test method for 'Transaction.getCardHolderName()'.
	 */
	public void testGetCardHolderName( )
	{
		CardHolder ch1 = new CardHolder( "Alan", 123456 );
		CardHolder ch2 = new CardHolder( "Daniel", 123456 );
		Transaction t1 = new Transaction( ch1 );
		Transaction t2 = new Transaction( ch2 );

		String nameExpected = new String( "Alan" );
		String nameActual = t1.getCardHolderName( );
		Assert.assertEquals( nameExpected, nameActual );

		nameExpected = new String( "Daniel" );
		nameActual = t2.getCardHolderName( );
		Assert.assertEquals( nameExpected, nameActual );
	}

	/**
	 * Test method for 'Transaction.getPoints()' with a purchase transaction and
	 * POINTS_FACTOR = 1.
	 */
	public void testGetPointsPurchaseNoBonus( )
	{
		CardHolder ch = new CardHolder( "Alan", 123456 );
		Transaction t = new Transaction( ch );

		Product p1 = new Product( "Product 1", 9.99, 50 );
		Product[] pList = new Product[] { p1 };
		AirKM.POINTS_FACTOR = 1;

		t.purchase( pList );

		int pointsExpected = 99;
		int pointsActual = t.getPoints( );
		Assert.assertEquals( pointsExpected, pointsActual );
	}

	/**
	 * Test method for 'Transaction.getPoints()' with a purchase transaction and
	 * POINTS_FACTOR = 3.
	 */
	public void testGetPointsPurchaseTripleBonus( )
	{
		CardHolder ch = new CardHolder( "Alan", 123456 );
		Transaction t = new Transaction( ch );

		Product p1 = new Product( "Product 1", 9.99, 50 );
		Product[] pList = new Product[] { p1 };
		AirKM.POINTS_FACTOR = 3;

		t.purchase( pList );

		int pointsExpected = 297;
		int pointsActual = t.getPoints( );
		Assert.assertEquals( pointsExpected, pointsActual );
	}

	/**
	 * Test method for 'Transaction.getPoints()' with a redemption transaction
	 * and POINTS_FACTOR = 1.
	 */
	public void testGetPointsRedemptionNoBonus( )
	{
		CardHolder ch = new CardHolder( "Alan", 123456 );
		ch.setNumberOfPoints( 50000 );
		Transaction t = new Transaction( ch );

		Product p1 = new Product( "Product 1", 1.23, 50 );
		Product p2 = new Product( "Product 2", 4.56, 75 );
		Product[] pList = new Product[] { p1, p2 };
		AirKM.POINTS_FACTOR = 1;

		t.redeemPoints( pList );

		int pointsExpected = 125;
		int pointsActual = t.getPoints( );
		Assert.assertEquals( pointsExpected, pointsActual );
	}

	/**
	 * Test method for 'Transaction.getPoints()' with a redemption transaction
	 * and POINTS_FACTOR = 3 (which shouldn't affect the result).
	 */
	public void testGetPointsRedemptionTripleBonus( )
	{
		CardHolder ch = new CardHolder( "Alan", 123456 );
		ch.setNumberOfPoints( 50000 );
		Transaction t = new Transaction( ch );

		Product p1 = new Product( "Product 1", 1.23, 50 );
		Product p2 = new Product( "Product 2", 4.56, 75 );
		Product[] pList = new Product[] { p1, p2 };
		AirKM.POINTS_FACTOR = 3;

		t.redeemPoints( pList );

		int pointsExpected = 125;
		int pointsActual = t.getPoints( );
		Assert.assertEquals( pointsExpected, pointsActual );
	}

	/**
	 * Test method for 'Transaction.getAmountPaid()'.
	 */
	public void testGetAmountPaid( )
	{
		CardHolder ch = new CardHolder( "Alan", 123456 );
		ch.setNumberOfPoints( 50000 );
		Transaction t = new Transaction( ch );

		double costExpected = 0.0;
		double costActual = t.getAmountPaid( );
		Assert.assertEquals( costExpected, costActual, 0.0001 );

		Product p1 = new Product( "Product 1", 1.23, 50 );
		Product p2 = new Product( "Product 2", 4.56, 75 );
		Product[] pList = new Product[] { p1, p2 };

		t.purchase( pList );

		costExpected = 5.79;
		costActual = t.getAmountPaid( );
		Assert.assertEquals( costExpected, costActual, 0.0001 );
	}

	/**
	 * Test method for 'Transaction.purchase(Product[])' with one item.
	 */
	public void testPurchase1( )
	{
		CardHolder ch = new CardHolder( "Alan", 123456 );
		ch.setNumberOfPoints(0);
		Transaction t = new Transaction( ch );

		Product p1 = new Product( "Product 1", 9.99, 50 );
		Product[] pList = new Product[] { p1 };
		AirKM.POINTS_FACTOR = 1;

		int categoryExpected = Transaction.INCOMPLETE;
		int categoryActual = t.getCategory();
		Assert.assertEquals(categoryExpected, categoryActual);
		
		boolean success = t.purchase( pList );

		boolean expected = true;
		boolean actual = success;
		Assert.assertEquals(expected, actual);
		
		categoryExpected = Transaction.PURCHASE;
		categoryActual = t.getCategory();
		Assert.assertEquals(categoryExpected, categoryActual);
		
		double paidExpected = 9.99;
		double paidActual = t.getAmountPaid();
		Assert.assertEquals(paidExpected, paidActual, 0.0001);
		
		int pointsExpected = 99;
		int pointsActual = t.getPoints();
		Assert.assertEquals(pointsExpected, pointsActual);
		
		expected = true;
		actual = CardHolder.enoughPoints(ch, pointsExpected);
		Assert.assertEquals(expected, actual);
		
		expected = false;
		actual = CardHolder.enoughPoints(ch, pointsExpected + 1);
		Assert.assertEquals(expected, actual);
	}

	/**
	 * Test method for 'Transaction.purchase(Product[])' with 2 items.
	 */
	public void testPurchase2( )
	{
		CardHolder ch = new CardHolder( "Alan", 123456 );
		ch.setNumberOfPoints(0);
		Transaction t = new Transaction( ch );

		Product p1 = new Product( "Product 1", 4.44, 50 );
		Product p2 = new Product( "Product 2", 5.55, 90 );
		Product[] pList = new Product[] { p1, p2 };
		AirKM.POINTS_FACTOR = 1;

		int categoryExpected = Transaction.INCOMPLETE;
		int categoryActual = t.getCategory();
		Assert.assertEquals(categoryExpected, categoryActual);
		
		boolean success = t.purchase( pList );

		boolean expected = true;
		boolean actual = success;
		Assert.assertEquals(expected, actual);
		
		double paidExpected = 9.99;
		double paidActual = t.getAmountPaid();
		Assert.assertEquals(paidExpected, paidActual, 0.0001);	
		
		categoryExpected = Transaction.PURCHASE;
		categoryActual = t.getCategory();
		Assert.assertEquals(categoryExpected, categoryActual);
		
		int pointsExpected = 99;
		int pointsActual = t.getPoints();
		Assert.assertEquals(pointsExpected, pointsActual);
		
		expected = true;
		actual = CardHolder.enoughPoints(ch, pointsExpected);
		Assert.assertEquals(expected, actual);
		
		expected = false;
		actual = CardHolder.enoughPoints(ch, pointsExpected + 1);
		Assert.assertEquals(expected, actual);
	}

	/**
	 * Test method for 'Transaction.purchase(Product[])' with triple points.
	 */
	public void testPurchase3( )
	{
		CardHolder ch = new CardHolder( "Alan", 123456 );
		ch.setNumberOfPoints(0);
		Transaction t = new Transaction( ch );

		Product p1 = new Product( "Product 1", 4.44, 50 );
		Product p2 = new Product( "Product 2", 5.55, 90 );
		Product[] pList = new Product[] { p1, p2 };
		AirKM.POINTS_FACTOR = 3;

		int categoryExpected = Transaction.INCOMPLETE;
		int categoryActual = t.getCategory();
		Assert.assertEquals(categoryExpected, categoryActual);
		
		boolean success = t.purchase( pList );

		boolean expected = true;
		boolean actual = success;
		Assert.assertEquals(expected, actual);
		
		double paidExpected = 9.99;
		double paidActual = t.getAmountPaid();
		Assert.assertEquals(paidExpected, paidActual, 0.0001);	
		
		categoryExpected = Transaction.PURCHASE;
		categoryActual = t.getCategory();
		Assert.assertEquals(categoryExpected, categoryActual);
		
		int pointsExpected = 297;
		int pointsActual = t.getPoints();
		Assert.assertEquals(pointsExpected, pointsActual);
		
		expected = true;
		actual = CardHolder.enoughPoints(ch, pointsExpected);
		Assert.assertEquals(expected, actual);
		
		expected = false;
		actual = CardHolder.enoughPoints(ch, pointsExpected + 1);
		Assert.assertEquals(expected, actual);
	}

	/**
	 * Test method for 'Transaction.purchase(Product[])', where purchase
	 * is called on an already completed transaction.
	 */
	public void testPurchase4( )
	{
		CardHolder ch = new CardHolder( "Alan", 123456 );
		ch.setNumberOfPoints(0);
		Transaction t = new Transaction( ch );

		Product p1 = new Product( "Product 1", 4.44, 50 );
		Product p2 = new Product( "Product 2", 5.55, 90 );
		Product[] pList = new Product[] { p1, p2 };
		AirKM.POINTS_FACTOR = 3;

		int categoryExpected = Transaction.INCOMPLETE;
		int categoryActual = t.getCategory();
		Assert.assertEquals(categoryExpected, categoryActual);
		
		boolean success = t.purchase( pList );

		boolean expected = true;
		boolean actual = success;
		Assert.assertEquals(expected, actual);

		Product p3 = new Product( "Product 3", 12.34, 50 );
		Product[] pList2 = new Product[] { p3 };

		success = t.purchase(pList2);
		
		expected = false;
		actual = success;
		Assert.assertEquals(expected, actual);
		
		double paidExpected = 9.99;
		double paidActual = t.getAmountPaid();
		Assert.assertEquals(paidExpected, paidActual, 0.0001);	
		
		categoryExpected = Transaction.PURCHASE;
		categoryActual = t.getCategory();
		Assert.assertEquals(categoryExpected, categoryActual);
		
		int pointsExpected = 297;
		int pointsActual = t.getPoints();
		Assert.assertEquals(pointsExpected, pointsActual);
		
		expected = true;
		actual = CardHolder.enoughPoints(ch, pointsExpected);
		Assert.assertEquals(expected, actual);
		
		expected = false;
		actual = CardHolder.enoughPoints(ch, pointsExpected + 1);
		Assert.assertEquals(expected, actual);
	}

	/**
	 * Test method for 'Transaction.redeemPoints(Product[])' with 2 items.
	 */
	public void testRedeemPoints1( )
	{
		CardHolder ch = new CardHolder( "Alan", 123456 );
		ch.setNumberOfPoints(50);
		Transaction t = new Transaction( ch );

		Product p1 = new Product( "Product 1", 9.99, 50 );
		Product[] pList = new Product[] { p1 };

		int categoryExpected = Transaction.INCOMPLETE;
		int categoryActual = t.getCategory();
		Assert.assertEquals(categoryExpected, categoryActual);
		
		boolean success = t.redeemPoints(pList);

		boolean expected = true;
		boolean actual = success;
		Assert.assertEquals(expected, actual);
		
		categoryExpected = Transaction.REDEMPTION;
		categoryActual = t.getCategory();
		Assert.assertEquals(categoryExpected, categoryActual);
		
		double paidExpected = 0.00;
		double paidActual = t.getAmountPaid();
		Assert.assertEquals(paidExpected, paidActual, 0.0001);
		
		int pointsExpected = 50;
		int pointsActual = t.getPoints();
		Assert.assertEquals(pointsExpected, pointsActual);
		
		expected = true;
		actual = CardHolder.enoughPoints(ch, 0);
		Assert.assertEquals(expected, actual);
		
		expected = false;
		actual = CardHolder.enoughPoints(ch, 1);
		Assert.assertEquals(expected, actual);

	}

	/**
	 * Test method for 'Transaction.redeemPoints(Product[])' with 1 item.
	 */
	public void testRedeemPoints2( )
	{
		CardHolder ch = new CardHolder( "Alan", 123456 );
		ch.setNumberOfPoints(140);
		Transaction t = new Transaction( ch );

		Product p1 = new Product( "Product 1", 4.44, 50 );
		Product p2 = new Product( "Product 2", 5.55, 90 );
		Product[] pList = new Product[] { p1, p2 };

		int categoryExpected = Transaction.INCOMPLETE;
		int categoryActual = t.getCategory();
		Assert.assertEquals(categoryExpected, categoryActual);
		
		boolean success = t.redeemPoints(pList);

		boolean expected = true;
		boolean actual = success;
		Assert.assertEquals(expected, actual);
		
		categoryExpected = Transaction.REDEMPTION;
		categoryActual = t.getCategory();
		Assert.assertEquals(categoryExpected, categoryActual);
		
		double paidExpected = 0.00;
		double paidActual = t.getAmountPaid();
		Assert.assertEquals(paidExpected, paidActual, 0.0001);
		
		int pointsExpected = 140;
		int pointsActual = t.getPoints();
		Assert.assertEquals(pointsExpected, pointsActual);
		
		expected = true;
		actual = CardHolder.enoughPoints(ch, 0);
		Assert.assertEquals(expected, actual);
		
		expected = false;
		actual = CardHolder.enoughPoints(ch, 1);
		Assert.assertEquals(expected, actual);

	}

	/**
	 * Test method for 'Transaction.redeemPoints(Product[])' with
	 * insufficient points.
	 */
	public void testRedeemPoints3( )
	{
		CardHolder ch = new CardHolder( "Alan", 123456 );
		ch.setNumberOfPoints(139);
		Transaction t = new Transaction( ch );

		Product p1 = new Product( "Product 1", 4.44, 50 );
		Product p2 = new Product( "Product 2", 5.55, 90 );
		Product[] pList = new Product[] { p1, p2 };

		int categoryExpected = Transaction.INCOMPLETE;
		int categoryActual = t.getCategory();
		Assert.assertEquals(categoryExpected, categoryActual);
		
		boolean success = t.redeemPoints(pList);

		boolean expected = false;
		boolean actual = success;
		Assert.assertEquals(expected, actual);
		
		categoryExpected = Transaction.INCOMPLETE;
		categoryActual = t.getCategory();
		Assert.assertEquals(categoryExpected, categoryActual);
		
		double paidExpected = 0.00;
		double paidActual = t.getAmountPaid();
		Assert.assertEquals(paidExpected, paidActual, 0.0001);
		
		int pointsExpected = 0;
		int pointsActual = t.getPoints();
		Assert.assertEquals(pointsExpected, pointsActual);	
	}

	/**
	 * Test method for 'Transaction.purchase(Product[])', where purchase
	 * is called on an already completed transaction.
	 */
	public void testRedeemPoints4( )
	{
		CardHolder ch = new CardHolder( "Alan", 123456 );
		ch.setNumberOfPoints(200);
		Transaction t = new Transaction( ch );

		Product p1 = new Product( "Product 1", 4.44, 50 );
		Product p2 = new Product( "Product 2", 5.55, 90 );
		Product[] pList = new Product[] { p1, p2 };
		AirKM.POINTS_FACTOR = 3;

		int categoryExpected = Transaction.INCOMPLETE;
		int categoryActual = t.getCategory();
		Assert.assertEquals(categoryExpected, categoryActual);
		
		boolean success = t.purchase( pList );

		boolean expected = true;
		boolean actual = success;
		Assert.assertEquals(expected, actual);

		Product p3 = new Product( "Product 3", 12.34, 50 );
		Product[] pList2 = new Product[] { p3 };

		success = t.redeemPoints(pList2);
		
		expected = false;
		actual = success;
		Assert.assertEquals(expected, actual);
		
		double paidExpected = 9.99;
		double paidActual = t.getAmountPaid();
		Assert.assertEquals(paidExpected, paidActual, 0.0001);	
		
		categoryExpected = Transaction.PURCHASE;
		categoryActual = t.getCategory();
		Assert.assertEquals(categoryExpected, categoryActual);
		
		int pointsExpected = 297;
		int pointsActual = t.getPoints();
		Assert.assertEquals(pointsExpected, pointsActual);
		
		expected = true;
		actual = CardHolder.enoughPoints(ch, pointsExpected + 200);
		Assert.assertEquals(expected, actual);
		
		expected = false;
		actual = CardHolder.enoughPoints(ch, pointsExpected + 201);
		Assert.assertEquals(expected, actual);
	}

	/**
	 * Test method for 'Transaction.toString()' with a purchase transaction.
	 */
	public void testToStringPurchase( )
	{
		CardHolder ch = new CardHolder( "Alan", 123456 );
		Transaction t = new Transaction( ch );

		Product p1 = new Product( "Product 1", 1.21, 50 );
		Product p2 = new Product( "Product 2", 4.50, 75 );
		Product[] pList = new Product[] { p1, p2 };
		AirKM.POINTS_FACTOR = 2;

		t.purchase( pList );

		String result = t.toString( );

		boolean expected = true;
		boolean actual = result.contains( "5.71" ); // cost
		Assert.assertEquals( expected, actual );

		expected = true;
		actual = result.contains( "114" ); // number of points
		Assert.assertEquals( expected, actual );

		expected = true;
		actual = result.contains( "Product 1" ); // product name
		Assert.assertEquals( expected, actual );

		expected = true;
		actual = result.contains( "Product 2" ); // product name
		Assert.assertEquals( expected, actual );

		expected = true;
		actual = result.contains( "Alan" ); // card holder
		Assert.assertEquals( expected, actual );

		expected = true;
		actual = result.toLowerCase( ).contains( "purchase" ); // transaction
																// type
		Assert.assertEquals( expected, actual );

		expected = true;
		actual = result.toLowerCase( ).contains( "obtained" ); // transaction
																// type
		Assert.assertEquals( expected, actual );
	}

	/**
	 * Test method for 'Transaction.toString()' with a redemption transaction.
	 */
	public void testToStringRedemption( )
	{
		CardHolder ch = new CardHolder( "Daniel", 654321 );
		ch.setNumberOfPoints( 50000 );
		Transaction t = new Transaction( ch );

		Product p1 = new Product( "Product 3", 1.21, 50 );
		Product p2 = new Product( "Product 4", 4.50, 75 );
		Product[] pList = new Product[] { p1, p2 };
		AirKM.POINTS_FACTOR = 2;

		t.redeemPoints( pList );

		String result = t.toString( );

		boolean expected = true;
		boolean actual = result.contains( "125" ); // number of points
		Assert.assertEquals( expected, actual );

		expected = true;
		actual = result.contains( "Product 3" ); // product name
		Assert.assertEquals( expected, actual );

		expected = true;
		actual = result.contains( "Product 4" ); // product name
		Assert.assertEquals( expected, actual );

		expected = true;
		actual = result.contains( "Daniel" ); // card holder
		Assert.assertEquals( expected, actual );

		expected = true;
		actual = result.toLowerCase( ).contains( "redemption" ); // transaction
																	// type
		Assert.assertEquals( expected, actual );

		expected = true;
		actual = result.toLowerCase( ).contains( "used" ); // transaction type
		Assert.assertEquals( expected, actual );
	}

	/**
	 * Test method for 'Transaction.toString()' with a redemption transaction.
	 */
	public void testToStringIncomplete( )
	{
		CardHolder ch = new CardHolder( "Grace", 451263 );
		Transaction t = new Transaction( ch );

		String result = t.toString( );

		boolean expected = true;
		boolean actual = result.toLowerCase()
			.contains( "transaction incomplete" ); // number of points
		Assert.assertEquals( expected, actual );
	}

}
